/* Copyright 2013 Robert Edward Steckroth II <RobertSteckroth@gmail.com> Bust0ut, Surgemcgee

* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; version 3.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

import QtQuick 2.0
import Ubuntu.Components 0.1

Item {
    property alias tab_id: tab_id_child
    NumberAnimation { id: animateOpacitySubTabs; target: tab_id_child; properties: "opacity"; from: 1.0; to: 0.6; duration: 40 }

   Rectangle {
        id: tab_id_child
        objectName: "subTabsBar"
        anchors.left: parent.left
        anchors.right: parent.right
        height: parent.height
        radius: 2
        color: "grey"
        gradient: Gradient {
            GradientStop { position: 0; color: "#676B6F" }
            GradientStop { position: 0.5; color: "#8B9299" }
            GradientStop { position: 1; color: "#676B6F" }
        }
        MouseArea {
            anchors.fill: parent
            property real mouse_in_box: 0
            onPressed: {
                currentPage.mainViewCurrent.snapMode = ListView.NoSnap // We want smooth scrolling
                mouse_in_box = mouseY // This stores the mouse position inside the tab bar. It avoids jumpy-ness when refa is calculated
                animateOpacitySubTabs.from = 1.0
                animateOpacitySubTabs.to = 0.7
                animateOpacitySubTabs.start()
            }
            onReleased: {
                currentPage.mainViewCurrent.snapMode = ListView.SnapPosition // Turn ratchet back on
                currentRectSizer.children[sub_tab_index].opacity = 1.0
                animateOpacitySubTabs.from = 0.7
                animateOpacitySubTabs.to = 1.0
                animateOpacitySubTabs.start()
            }
            onMouseYChanged: {
                refa = (mainProgram.height/((mouseY+(mainProgram.height/refa))-mouse_in_box)).toFixed(4)
                if ( mainProgram.height/refa > mainProgram.height-rootTabs.header.height-rootTabs.toolbar.height-100 ) // Not too far down
                     refa = (mainProgram.height/(mainProgram.height-rootTabs.header.height-rootTabs.toolbar.height-100)).toFixed(4)
                else if ( mainProgram.height/refa < sortPage.height+subtabsPage.height ) // Not past the sortTabs bar
                          refa = (mainProgram.height/(sortPage.height+subtabsPage.height)).toFixed(4)
                currentPage.mainViewCurrent.positionViewAtIndex(currentPage.mainViewCurrent.currentIndex, ListView.Contain)
                detailsPage.detailViewCurrent.positionViewAtIndex(currentPage.mainViewCurrent.currentIndex, ListView.Center)
                chartsPage.wipeCanvas() // Clear the canvas
                onPaintCanvas.start()  // Delay the re-draw with this timer (it looks better)
            }
        }
        UbuntuShape {
            id: chartsTab
            objectName: "chartsTab"
            color: "#115980"
            gradientColor: "#12384D"
            radius: "small"
            width: units.gu(6)
            height: parent.height
            anchors.left: parent.left
            anchors.leftMargin: units.gu(1)

            Text {
                anchors.centerIn: parent
                text: i18n.tr("Charts")
                font.pixelSize: FontUtils.sizeToPixels("small")
                color: "#f1f1f1"
            }
            MouseArea {
                anchors.fill: parent
                onClicked: control_tab(1, false)
            }
        }
        UbuntuShape {
            id: detailsTab
            objectName: "detailsTab"
            color: "#115980"
            gradientColor: "#12384D"
            radius: "small"
            width: units.gu(6)
            height: parent.height
            anchors.horizontalCenter: parent.horizontalCenter
            Text {
                anchors.centerIn: parent
                text: i18n.tr("Details")
                font.pixelSize: FontUtils.sizeToPixels("small")
                color: "#f1f1f1"
            }
            MouseArea {
                anchors.fill: parent
                onClicked: control_tab(2, false)
            }
        }
        UbuntuShape {
            id: newsTab
            objectName: "newsTab"
            color: "#115980"
            gradientColor: "#12384D"
            radius: "small"
            width: units.gu(6)
            height: parent.height
            anchors.right: parent.right
            anchors.rightMargin: units.gu(1)
            Text {
                anchors.centerIn: parent
                text: i18n.tr("News")
                font.pixelSize: FontUtils.sizeToPixels("small")
                color: "#f1f1f1"
            }
            MouseArea {
                anchors.fill: parent
                onClicked: control_tab(3, false)
            }
        }
    }
}

