/* Copyright 2013 Robert Edward Steckroth II <RobertSteckroth@gmail.com> Bust0ut, Surgemcgee

* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; version 3.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

.pragma library

var Graph = function (args) {

    args = args || {}
    this.defaults = {}
    this.options = {}

    this.options.topPadding = this.defaults['topPadding'] = args['topPadding'] || 0
    this.options.rightPadding = this.defaults['rightPadding'] = args['rightPadding'] || 0
    this.options.bottomPadding = this.defaults['bottomPadding'] = args['bottomPadding'] || 0
    this.options.leftPadding = this.defaults['leftPadding'] = args['leftPadding'] || 0
    this.options.width = this.defaults['width'] = args['width'] || 300
    this.options.height = this.defaults['height'] = args['height'] || 300
    this.options.dateTextColor = this.defaults['dateTextColor'] = args['dateTextColor'] || "#333333"
    this.options.volumeLineColor = this.defaults['volumeLineColor'] = args['volumeLineColor'] || "blue"
    this.options.tradePointColor = this.defaults['tradePointColor'] = args['tradePointColor'] || "darkgrey"
    // These will be saved into the local storage
    this.options.showDayRange = this.defaults['showDayRange'] = args['showDayRange'] || "no" // Can not use boolean in our localStorage

    var chart = []
    this.addChart = function (b) {
         chart.push(b)
    }

    this.init = function (ctx) {
        this.ctx = ctx
    }

    this.renderAll = function () {
        if ( chart.length > 0 )
             for ( var i = 0; i < chart.length; i++ )
                 chart[i].draw()
    }

    this.render = function (i) {
        if ( chart[i] )
            chart[i].draw()
    }

    this.getChartSetting = function(num, arg) {
        if ( chart.length > 0 )
        return chart[num][arg]
    }

    this.wipeAll = function () {
        if ( chart.length > 0 )
             for ( var i = 0; i < chart.length; i++ )
                 chart[i].wipe();
    }

    this.contextSetSave = function(args) {
        for ( var x in args )
            if ( this.ctx && this.ctx[x] )
                this.ctx[x] = args[x]
    }


    this.getDayInfo = function (i, mX) {
        if ( chart[i] )
            return chart[i].getDay(mX)
    }
}

var Chart = function (g, listmodel, args) {
    args = args || {}
    this.listModel = listmodel
    this.graph = g
    this.past_len = 0
    this.space_average = 0
    this.volume_adjust = 0
    this.min_value = 0
    this.volume_size = 0
    this.value_adjust = 0
    this.chartLineColor = args['chartLineColor'] || "black"
    this.volume_bar_size = 0.13 // Percent of the graph height
    this.chartLineTop = 0
    this.chartLineBottom = 0
    this.register();
}

Chart.prototype = {

    register: function () {
          this.graph.addChart(this);
    },
    wipe: function() {
        if ( this.graph.ctx )
            this.graph.ctx.clearRect(0, 0, this.graph.options.width, this.graph.options.height)
    },
    getDay: function(mX) {

        var dayInfo = []
        var graphBottom = this.graph.options.bottomPadding+((this.graph.ctx.measureText('M').width+5)*2)
        var curr = Math.round((mX-this.graph.options.leftPadding) / this.space_average)
        if ( curr > this.past_len )
            curr = this.past_len
        else if ( curr < 0 )
            curr = 0

        if ( typeof(this.listModel.get(this.past_len-curr)) === "undefined" ) return false
        // 0 is listModel index
        dayInfo.push(curr)
        // 1 is x position
        dayInfo.push((curr * this.space_average)+this.graph.options.leftPadding)
        // 2 is y position
        dayInfo.push(Math.round((this.graph.options.height-this.volume_size-graphBottom)-(parseFloat(this.listModel.get(this.past_len-curr).close-this.min_value)*this.value_adjust)))
        return dayInfo
    },
    draw: function () {

        if ( ! this.graph.ctx ) return false

        this.char_pixel_size = this.graph.ctx.measureText('M').width+5 // S very close calculation of the text chartacter height
        //this.char_pixel_height = this.graph.ctx.measureText('M').width
        //this.char_pixel_width = this.graph.ctx.measureText('8').width

        this.past_len = this.listModel.count-1
        this.space_average = parseFloat(((this.graph.options.width-(this.graph.options.leftPadding+this.graph.options.rightPadding))/this.past_len).toFixed(3))

        if ( this.past_len <= 0 ) {
             this.graph.ctx.clearRect(0, 0, this.graph.options.width, this.graph.options.height)
             return
        }

        var sn = {}
        // mass initialization here
        sn.max_value = sn.day_high = sn.day_close = sn.day_low = sn.max_volume = sn.day_volume = sn.max_close_high = sn.price_length = sn.max_close_low_beg = sn.max_close_high_beg = 0
        sn.max_close_low = 10000000 // Higher than any stock can be
        this.min_value = 10000000 // This is a "this" so it can be used in the getDay member function

        for ( var x = this.past_len; x >= 0;  x-- ) {
            sn.day_volume = parseFloat(this.listModel.get(x).volume)
            sn.day_close = parseFloat(this.listModel.get(x).close)
            if ( sn.day_volume > sn.max_volume ) sn.max_volume = sn.day_volume
            if ( sn.day_close > sn.max_close_high ) sn.max_close_high = sn.day_close // Get the largest trade value
            if ( sn.day_close < sn.max_close_low ) sn.max_close_low = sn.day_close   // Get the smallest trade value

            if ( this.graph.options.showDayRange === "yes" ) {
                sn.day_high = parseFloat(this.listModel.get(x).high)
                sn.day_low = parseFloat(this.listModel.get(x).low)
                // These are use when the range bars are active to fit the chart into the demensions
                if ( sn.day_low < this.min_value ) this.min_value = sn.day_low  // Get the smallest trade range value
                if ( sn.day_high > sn.max_value ) sn.max_value = sn.day_high  // Get the largest trade range value

                sn.price_length = (parseFloat(sn.max_close_low).toFixed(2).length-1) * this.char_pixel_size  // This caclulates the length in pixels of the left trade amount text
                if ( ((this.past_len-x)*this.space_average)+this.graph.options.leftPadding < sn.price_length ) {  // This sets the text down from the day range if nessesary
                    if ( this.min_value < sn.max_close_low  ) sn.max_close_low_beg = this.min_value // We do not want the range bar to overlap the text
                    if ( sn.max_value > sn.max_close_high_beg ) sn.max_close_high_beg = sn.max_value
                }
            }
            else {
                this.min_value = sn.max_close_low
                sn.max_value = sn.max_close_high
            }
        }

        if ( this.graph.options.showDayRange === "yes" ) {
            if ( sn.max_close_high_beg > sn.max_close_high )
                sn.max_close_high_beg = sn.max_close_high_beg-sn.max_close_high
            else sn.max_close_high_beg = 0
            if ( sn.max_close_low_beg < sn.max_close_low )
                sn.max_close_low_beg = sn.max_close_low-sn.max_close_low_beg
            else sn.max_close_low_beg = 0
        }

        this.min_value = parseFloat(this.min_value).toFixed(2)

        for ( var o in sn ) // We wnat all 2 decimal rounded numbers
            sn[o] = parseFloat(sn[o]).toFixed(2)

        this.volume_size = Math.round(this.graph.options.height * this.volume_bar_size)
        this.volume_adjust = (this.volume_size/sn.max_volume).toFixed(12)

        var graph_bottom = this.graph.options.bottomPadding+this.char_pixel_size*2
        this.value_adjust = (this.graph.options.height-this.graph.options.topPadding-this.volume_size-graph_bottom-this.char_pixel_size) / (sn.max_value-this.min_value)

        this.graph.ctx.clearRect(0,0,this.graph.options.width, this.graph.options.height)

        this.graph.ctx.lineWidth = 1
        sn.day_high = 0 //Re-using this varible here
        sn.day_low = 0  //Re-using this varible here
        sn.day_close = 0//Re-using this varible here
        var day_open = 0, text_pos = 0, date_text = ""

        var oneShot = false, dateSpacing, dateTextLength, dateFormat = ''

        var dateArray = this.listModel.get(this.past_len).date.split('-')
        if ( parseInt(dateArray[1]) < parseInt(new Date().getMonth()) )  // This will find if the chart goes back from this month and year for text display
            dateFormat = 'month'
        if ( parseInt(dateArray[0]) < parseInt(new Date().getFullYear()) )
            dateFormat = 'year'

        var xv = this.graph.options.leftPadding, ff = false

        this.graph.ctx.beginPath()
        xv = this.graph.options.leftPadding
        this.graph.ctx.moveTo(xv, this.graph.options.height-this.volume_size-graph_bottom )

        for ( var i = this.past_len; i >= 0;  i-- ) {

            // Graph trade line
            if ( !ff ) ff = (this.graph.options.height-this.volume_size-graph_bottom)-(parseFloat(this.listModel.get(i).close-this.min_value)*this.value_adjust).toFixed(2)
            ff = (this.graph.options.height-this.volume_size-graph_bottom)-(parseFloat(this.listModel.get(i).close-this.min_value)*this.value_adjust).toFixed(2)

            this.graph.ctx.lineTo(xv, ff )
            this.graph.ctx.strokeStyle = this.chartLineColor
            this.graph.ctx.stroke();
            // ----------------------------
            xv += this.space_average
        }

        this.graph.ctx.lineTo(xv-this.space_average, this.graph.options.height-this.volume_size-graph_bottom )

        this.graph.ctx.stroke()

        if ( this.graph.options.showDayRange !== "yes" ) {
            this.graph.ctx.closePath()
            this.graph.ctx.fillStyle = this.graph.ctx.createLinearGradient(this.graph.options.width-this.graph.options.rightPadding, 0,
            this.graph.options.width-this.graph.options.rightPadding, this.graph.options.height-this.volume_size-graph_bottom).addColorStop(0,"#A1A1A1").addColorStop(1,"#ededed")
            this.graph.ctx.fill()
        }

        xv = this.graph.options.leftPadding

        for ( var i = this.past_len; i >= 0;  i-- ) {
            day_open = parseFloat(this.listModel.get(i).open)
            sn.day_close = parseFloat(this.listModel.get(i).close)

            // Graph day range
            if ( this.graph.options.showDayRange === "yes" ) {
                sn.day_low = (this.graph.options.height-this.volume_size-graph_bottom)-(parseFloat(this.listModel.get(i).low-this.min_value)*this.value_adjust).toFixed(2)
                sn.day_high = (this.graph.options.height-this.volume_size-graph_bottom)-(parseFloat(this.listModel.get(i).high-this.min_value)*this.value_adjust).toFixed(2)
                this.graph.ctx.beginPath()
                if ( day_open < sn.day_close ) {  // Going up (green)
                    this.graph.ctx.strokeStyle = "#33512A"
                    this.graph.ctx.moveTo(xv, sn.day_low)
                    this.graph.ctx.lineTo(xv, sn.day_high)
                }
                else { this.graph.ctx.strokeStyle = "#741806"  // Going down (red)
                    this.graph.ctx.moveTo(xv, sn.day_high)
                    this.graph.ctx.lineTo(xv, sn.day_low)
                 }
                 this.graph.ctx.stroke()
            }
            //-----------------------------

            // Graph volume bar
            this.graph.ctx.beginPath();
            this.graph.ctx.strokeStyle = "#0C276D" // this.graph.options.volumeLineColor"
            this.graph.ctx.moveTo(xv, this.graph.options.height-graph_bottom+this.char_pixel_size)
            this.graph.ctx.lineTo(xv, this.graph.options.height-graph_bottom+this.char_pixel_size- (Math.round(parseFloat(this.listModel.get(i).volume)*this.volume_adjust)))
            this.graph.ctx.fillStyle = this.graph.options.volumeLineColor
            this.graph.ctx.stroke();
            // ----------------------------

            // Graph bottom date text
            this.graph.ctx.fillStyle = this.graph.options.dateTextColor

            dateArray = this.listModel.get(i).date.split('-')
            if ( !dateFormat ) {
                date_text = dateArray[2].toString()
                dateSpacing = 10
            }
            else if ( dateFormat === 'month' ) {
                date_text = dateArray[1]+"/"+dateArray[2]
                dateSpacing = 10
            }
            else if ( dateFormat === 'year' ) {
                //date_text = this.listModel.get(i).date.split('-')  // I wonder how thorough the gargage collection is in qt?
                //date_text = date_text[1]+"/"+date_text[2]+"/"+date_text[0].substr(2)
                date_text = dateArray[1]+"/"+dateArray[2]+"/"+dateArray[0].substr(2)
                dateSpacing = 10
            }

            dateTextLength = date_text.length*this.char_pixel_size
            if ( ( text_pos >= dateTextLength/4 && !oneShot ) || ( text_pos >= dateTextLength/2+dateSpacing && i*this.space_average > dateTextLength/4 ) ) {
                text_pos = 0
                oneShot = true
                this.graph.ctx.textBaseline = 'bottom'
                this.graph.ctx.fillText(date_text, xv-dateTextLength/4, this.graph.options.height)
            }
            // ----------------------------

            text_pos += this.space_average
            xv += this.space_average
        }


        this.graph.ctx.beginPath();
        this.graph.ctx.strokeStyle = this.graph.options.tradePointColor
        this.graph.ctx.fillStyle = this.graph.options.tradePointColor
        // The top chart line and trade text (high)
        this.chartLineTop = (this.graph.options.height-this.volume_size-graph_bottom)-((sn.max_close_high-this.min_value)*this.value_adjust).toFixed(2)
        this.chartLineBottom = (this.graph.options.height-this.volume_size-graph_bottom)-((sn.max_close_high-this.min_value)*this.value_adjust).toFixed(2)

        this.graph.ctx.moveTo(this.graph.options.leftPadding, this.chartLineTop)
        this.graph.ctx.lineTo(this.graph.options.width-this.graph.options.rightPadding, this.chartLineTop)
        this.graph.ctx.textBaseline = 'bottom'
        this.graph.ctx.fillText(sn.max_close_high, this.graph.options.leftPadding, this.chartLineTop-Math.round(sn.max_close_high_beg*this.value_adjust) )

        // The bottom chart line and trade text (low)
        this.chartLineBottom = (this.graph.options.height-this.volume_size-graph_bottom)-((sn.max_close_low-this.min_value)*this.value_adjust).toFixed(2)
        this.graph.ctx.moveTo(this.graph.options.leftPadding, this.chartLineBottom)
        this.graph.ctx.lineTo(this.graph.options.width-this.graph.options.rightPadding, this.chartLineBottom);
        this.graph.ctx.textBaseline = 'top'
        this.graph.ctx.fillText(sn.max_close_low, this.graph.options.leftPadding, this.chartLineBottom+Math.round(sn.max_close_low_beg*this.value_adjust)+1 )
        this.graph.ctx.stroke();

    } // Endo draw()

} // Endo Chart.prototype


var graph = new Graph({ topPadding: 20, rightPadding: 5, bottomPadding: 0, leftPadding: 5, showDayRange: "no",
                        dateTextColor: "#3D1D04", tradePointColor: "#434343", volumeLineColor: "#2C2832"
                      })

var newChart = function(listModel) {
    new Chart(graph, listModel, {chartLineColor: "#2C1706"})
}
