# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright 2013 Robert Edward Steckroth II <RobertSteckroth@gmail.com> Bust0ut, Surgemcgee

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authored by: Robert Steckroth, Surgemcgee Bust0ut <RobertSteckroth@gmail.com>


""" Tests for stock_ticker """

from __future__ import absolute_import

from testtools.matchers import Equals, NotEquals, Not, Is
from autopilot.matchers import Eventually
from stock_ticker.tests import StockTickerTestCase

class TestManagementPage(StockTickerTestCase):
    """Tests the StockTicker Management page functionality"""

    def setUp(self):
        super(TestManagementPage, self).setUp()
        self.assertThat(
            self.ubuntu_sdk.get_qml_view().visible, Eventually(Equals(True)))

    def tearDown(self):
        super(TestManagementPage, self).tearDown() 

       
    def test_add_remove_stock(self):
        """Tests adding a stock by typing it into the text feild, selecting it from the autocomplete drop down and clicking the add button in the listview."""

        tabs = self.app.select_single("Tabs", objectName="rootTabs")

        self.ubuntu_sdk.switch_to_tab(1)
        self.assertThat(tabs.selectedTabIndex, Eventually(Equals(1)))

        self.ubuntu_sdk.after_program_fully_loaded()
    
        add_text_feild = self.app.select_single('TextField', objectName='addStockTextField')        
      
        self.ubuntu_sdk.click_object(add_text_feild)

        self.ubuntu_sdk.autopilot.keyboard.press_and_release('End', delay=0.2)
        self.ubuntu_sdk.autopilot.keyboard.press_and_release(' ', delay=0.3)
        self.ubuntu_sdk.autopilot.keyboard.press_and_release('g', delay=0.3)
#        self.ubuntu_sdk.autopilot.keyboard.press_and_release('o', delay=0.3)
#        self.ubuntu_sdk.autopilot.keyboard.press_and_release('o', delay=0.3)
#        self.ubuntu_sdk.autopilot.keyboard.press_and_release('g', delay=0.3)

        add_text_popover = self.app.select_single('Popover', objectName='autoCompletePopup')
        self.assertThat(add_text_popover.visible, Eventually(Equals(True)))

        autocomplete_activity_indicator = add_text_popover.select_single('ActivityIndicator', objectName='autocompleteActivity')
        self.assertThat(autocomplete_activity_indicator.running, Eventually(Equals(False)))
                
        search_matches = add_text_popover.select_single('QQuickListView', objectName='matchListView')

        management_item = self.app.select_single('QQuickItem', objectName='managementAdd')
        management_listview = management_item.select_single('QQuickListView', objectName='mainViewAdd')

        add_list_count = int(management_listview.count)

        self.assertThat(search_matches.count, Eventually(Equals(10)))
        search_matches_list = search_matches.get_children()[0].get_children()

        for o in search_matches_list:
            search_match_name = o
            if search_match_name and hasattr(search_match_name, 'text'):
                if search_match_name.text == 'GOOG Google Inc. NASDAQ':
                    break

        self.assertThat(search_match_name.text, Eventually(Equals('GOOG Google Inc. NASDAQ')))

        self.ubuntu_sdk.click_object(search_match_name)

        self.assertThat(management_listview.count, Eventually(Equals(add_list_count+1))) # the listElement was added to the management_listview

        management_delegate = management_listview.get_children()[0].get_children()

        cmpy_index = -1
        for o in management_delegate:
            add_list_item = o.select_single('QQuickText', objectName='companyNameText')  
            cmpy_index += 1
            if add_list_item and hasattr(add_list_item, 'text'):
                if add_list_item.text == 'Google Inc.':
                    break

        self.assertThat(add_list_item.text, Eventually(Equals('Google Inc.')))

        remove_button = management_delegate[cmpy_index].select_single('QQuickItem', objectName='removeButton')
        self.ubuntu_sdk.click_object(remove_button)

 
        self.assertThat(management_listview.count, Eventually(Equals(add_list_count)))

















