# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright 2013 Robert Edward Steckroth II <RobertSteckroth@gmail.com> Bust0ut, Surgemcgee

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authored by: Robert Steckroth, Surgemcgee Bust0ut <RobertSteckroth@gmail.com>


""" Tests for stock_ticker """

from __future__ import absolute_import

from testtools.matchers import Equals, NotEquals, Not, Is
from autopilot.matchers import Eventually
from stock_ticker.tests import StockTickerTestCase

class TestToolbar(StockTickerTestCase):
    """Tests the Stock Ticker toolbar"""

    def setUp(self):
        super(TestToolbar, self).setUp()
        self.assertThat(
            self.ubuntu_sdk.get_qml_view().visible, Eventually(Equals(True)))

    def tearDown(self):
        super(TestToolbar, self).tearDown() # why doesn't this not exist in base test package class?

    def _open_toolbar(self):
        """Switches to the Watched tab and opens the toolbar"""

        tabs = self.app.select_single("Tabs", objectName="rootTabs")
        self.ubuntu_sdk.switch_to_tab(0)
        self.assertThat(tabs.selectedTabIndex, Eventually(Equals(0)))

        self.ubuntu_sdk.open_toolbar()

        toolbar = self.app.select_single("Toolbar")

        self.assertThat(toolbar.opened, Eventually(Equals(True)))

        return toolbar

        
    def test_application_reset(self):
        """Tests if the sub tabs bar is resizing and if the appication resets back to the defaults."""

        self.ubuntu_sdk.after_program_fully_loaded()

        toolbar = self._open_toolbar()
        toolbar_type = toolbar.select_single('ToolbarItems')
        toolbar_all = toolbar_type.select_single('QQuickRow').get_children()

        toolbar_item = {}       
        for o in toolbar_all:
            if o.objectName:
                toolbar_item[o.objectName] = o

        self.ubuntu_sdk.click_object(toolbar_item['resetData'])
        reset_dialog = self.app.select_single('Dialog', objectName='resetDataDialog').select_single('Button', objectName='resetConfirm')
        self.ubuntu_sdk.click_object(reset_dialog)

        top_window = self.app.select_single('QQuickItem', objectName='currentRectSizer')  # going to set this and then make sure it back to its default height
        top_window_height = top_window.height
        self.ubuntu_sdk.move_subtabs_bar_up()
        self.assertThat(top_window_height, NotEquals(top_window.height))


        self._open_toolbar()
        self.ubuntu_sdk.click_object(toolbar_item['resetData'])
        reset_dialog = self.app.select_single('Dialog', objectName='resetDataDialog').select_single('Button', objectName='resetConfirm') # re-assign due to the way component wrapped objects behave
        self.ubuntu_sdk.click_object(reset_dialog)

        self.assertThat(top_window_height, Equals(top_window.height))


       
























